// nbPluginAPI.h

#ifndef __NB_PLUGIN_API_H__
#define __NB_PLUGIN_API_H__

//*************************************************************************
// Header that define Plugin interface for use with newsbin.
//
//	On Startup, Newsbin will enumerate the DLL's on the "plugin" folder,
//	and use any that support this interface.
//
// NOTE:  Compatible with newsbin 4.32
//*************************************************************************

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _entrypoints				ENTRYPOINTS_V2;

typedef const void*						NBFILEDATA;
typedef struct _file_callbacks			FILECALLBACKS_V1;

typedef const void*						NBPOSTDATA;
typedef struct _post_callbacks			POSTCALLBACKS_V1;

typedef struct _record RECORD_T;

// Bit values that the plugin should return in pType when Newsbin calls
// PG_Init() to let Newsbin known which entrypoints the plugin wants
// Newsbin to call.
//
// The Plugin can create new "download records".
// Normally used in conjunction with PG_FILE_HANDLER
// If set, Newsbin will call PG_???
#define      PG_HEADER_SOURCE      0x00000001
//
// The Plugin wants to be called when every post becomes complete.
// If set Newsbin will call PG_PostFilter().
#define      PG_POST_FILTER         0x00000010
//
// The Plugin wants to be called when every file becomes complete.
// If set Newsbin will call PG_FileFilter().
#define      PG_FILE_FILTER         0x00000020
//
// The Plugin wants to be called before every download starts.
// If set Newsbin will call PG_PreDownload().
#define      PG_PRE_DOWNLOAD         0x00000100
//
// The Plugin wants to be called after every download completes.
// If set Newsbin will call PG_PostPownload().
#define      PG_POST_DOWNLOAD      0x00001000
//
// The Plugin wants to be called after every chunk is decoded.
// If set Newsbin will call PG_ChunkDownload().
#define      PG_CHUNK_DOWNLOAD      0x00010000
//
// The Plugin is interested in files with specific file extensions.
// If set Newsbin will call PG_Filename(), PG_Read(), and PG_Close().
#define      PG_FILE_HANDLER         0x00100000
//
// The Plugin wants to be passed a list of loaded posts so that it can
// select which ones to download.
// If set Newsbin will call PG_???
#define      PG_DOWNLOAD_FILTER   0x00200000


//
// Tells Newsbin that this plugin should have a PG_Options entry and that
//	it can be called to let the plugin display a configuration dialog. 
//
#define		PG_HAS_OPTIONS			0x00400000


typedef   unsigned long				PLUGIN_TYPE; // 32 bit value

typedef   unsigned long				VERSION_TYPE; // 32 bit value

#define      S_DATADONE            _HRESULT_TYPEDEF_(0x00001000L)

struct _entrypoints
{
  // The Plugin must initialise this to the size of the structure so
  // that Newsbin can check that it is the correct version.
  size_t			PG_nSize;

  // This is the first function that Newsbin calls.
  // The plugin should set *pType with values indicating which other
  // entry points the plugin has defined.
  // The plugin should record the p1 and p2 values for use when making
  // calbacks into Newsbin to obtain information.
  // The plugin should return S_OK if it is able to proceed,
  // or S_??? if not.
  HRESULT (WINAPI *PG_Init)(PLUGIN_TYPE* pType,
                            FILECALLBACKS_V1* p1,
                            POSTCALLBACKS_V1* p2);

  // This function is called by Newsbin to find out what the plugin wants
  // to call itself. Newsbin provides a buffer pointed by pszDescription
  // and nLength where the plugin should store its name.
  // The plugin should return S_OK if the name has been returned
  // or S_??? if not.
  HRESULT (WINAPI *PG_Description)(char* pszDescription,
                                   size_t nLength);

  // This function is called by Newsbin to find out what file extensions
  // the plugin is interested in. ??? This entry point is only called if
  // the plugin specified PG_FILE_HANDLER when PG_Init was called. Newsbin
  // provides a buffer pointed to by pszExtension and nLength where the
  // plugin should store a comma separated list of the file extensions it
  // is interested in. Each file extension should include the period.
  // The plugin should return S_OK if the file extensions were returned
  // or S_??? if not.
  HRESULT (WINAPI *PG_Extension)(char* pszExtension,
                                 size_t nLength);

  // This function is called by Newsbin to allow the plugin to display a
  // configuration dialogue. The plugin should create the dialogue using
  // the supplied hWnd as the parent window.
  // The plugin should return S_OK.
  HRESULT (WINAPI *PG_Options)(HWND hWnd);


  // These functions are called by Newsbin to let a plugin read the
  // contents of a file and let the plugin supply Newsbin with the details
  // of other posts that Newsbin should download.
  // PG_Filename is called first to supply the pluggin with the name of
  // the file that the plugin should process. The Plugin should return
  // S_OK if it wants Newsbin to proceed, or S_?? if not.
  // PG_Read will be called repeatedly by Newsbin and the plugin should
  // fill in the supplied structure with information about a post that
  // should be downloaded. The Plugin should  return S_OK if it has
  // filled in the structure, S_DATA_DONE if there is no more information
  // available, and S_??? if an error occurred.
  // PG_Close will be called by Newsbin once all data has been returned.
  // The plugin should clean up and resources it may have allocated and
  // return S_OK.
  HRESULT   (WINAPI *PG_Open)(const char* pszFilename);
  HRESULT   (WINAPI *PG_Read)(RECORD_T* pRecord);
  HRESULT   (WINAPI *PG_Close)(void);


  // This function is called by Newsbin before a download starts. This
  // function is only  called if the plugin specified PG_PREDOWNLOAD when
  // PG_Init() was called.
  // The plugin may obtain details about the post by calling the
  // CALLBACK_POINTS functions.
  // The plugin should return S_??? to indicate ???.
  HRESULT   (WINAPI *PG_PreDownload)(NBFILEDATA hFileData);

  // This function is called by Newsbin after a download completes. This
  // function is only  called if the plugin specified PG_POSTDOWNLOAD
  // when PG_Init() was called.
  // The plugin may obtain details about the post by calling the
  // CALLBACK_POINTS functions. The plugin should return S_??? to
  // indicate ???.
  HRESULT   (WINAPI *PG_PostDownload)(NBFILEDATA hFileData);

  // This function is called by Newsbin after a chunk has been decoded.
  // This function is only called if the plugin specified PG_CHUNKDOWNLOAD
  // when PG_Init() was called.
  // nIndex, pszData, and nLength specify (what?).
  // The plugin should return S_??? to indicate ???.
  HRESULT   (WINAPI *PG_ChunkDownload)(NBFILEDATA hFileData,size_t nIndex,
                                     const unsigned char* pszData,
                                     size_t nLength);

  // This function is called by Newsbin when a post becomes complete.
  // This function is only called if the plugin specified PG_POSTFILTER
  // when PG_Init() was called.
  // The plugin may obtain details about the post by calling the
  // CALLBACK_POSTS functions.
  // The plugin should return S_OK to let Newbin display the post or
  // E_ABORT to filter it out.
  HRESULT   (WINAPI *PG_PostFilter)(NBPOSTDATA hPostData);

  // This function is called by Newsbin when a file becomes complete. This
  // function is only called if the plugin specified PG_FILEFILTER when
  // PG_Init() was called.
  // The plugin may obtain details about the post by calling the
  // CALLBACK_POINTS functions.
  // The plugin should return S_??? to indicate ???.
  HRESULT   (WINAPI *PG_FileFilter)(NBFILEDATA hFileData);


  // This function is called by Newsbin when a file becomes complete. This
  // function is only called if the plugin specified PG_FILEFILTER when
  // PG_Init() was called.
  // The plugin may obtain details about the post by calling the
  // CALLBACK_POINTS functions.
  // The plugin should return S_??? to indicate ???.
  HRESULT   (WINAPI *PG_Version)(VERSION_TYPE* PluginVersion,VERSION_TYPE nNewsbinVersion);

};

// These functions may be called from PG_PostFilter to obtain information
// about a post.
struct _post_callbacks
{
  size_t         PG_nSize;

  // "Subject:" header from post
  const char* (WINAPI *CP_Subject)       (NBPOSTDATA hPostData);

  // "From:" header from post
  const char* (WINAPI *CP_From)          (NBPOSTDATA hPostData);

  // Size of post
  size_t      (WINAPI *CP_Size)          (NBPOSTDATA hPostData);

  // Total number of articles from a split file
  size_t      (WINAPI *CP_MaxRecords)    (NBPOSTDATA hPostData);

  // Article number of the current article from a split file
  size_t      (WINAPI *CP_CurrentRecords)(NBPOSTDATA hPostData);

  // Date of article
  time_t      (WINAPI *CP_Date)          (NBPOSTDATA hPostData);
};

// These functions may be called from PG_FileFilter, PG_PreDownload, and
// PG_PostDownload to obtain information about a file.
struct _file_callbacks
{
  size_t         PG_nSize;

  // "Subject:" header from post
  const char* (WINAPI *CB_Subject)   (NBFILEDATA hFileData);

  // "From:" header from post
  const char* (WINAPI *CB_From)      (NBFILEDATA hFileData);

  // Filename if known (does not include path)
  const char* (WINAPI *CB_Filename)  (NBFILEDATA hFileData);

  // Size of file
  size_t      (WINAPI *CB_Size)      (NBFILEDATA hFileData);

  // Total number of articles from a split file
  size_t      (WINAPI *CB_MaxRecords)(NBFILEDATA hFileData);

  // 1 if all articles are available, 0 if not
  size_t      (WINAPI *CB_Complete)  (NBFILEDATA hFileData);

  // Date of article
  time_t      (WINAPI *CB_Date)      (NBFILEDATA hFileData);
};

// The plugin fills in the structure with information when Newsbin calls
// PG_Read().
struct _record
{
  const char*   pszSubject;
  const char*   pszGroup;
  const char*   pszServer;
  const char*   pszFrom;
  const char*   pszMessageId;
  size_t      nRecordDate;
  size_t      nRecordSize;
  size_t      nRecordNumber;
  size_t      nRecordIndex;
};

// This structure appears to be completely unused.
#ifndef _XOVERDEF
#define _XOVERDEF
typedef struct   _xoverinfo
{
  size_t   m_nRecordNumber;
  size_t   m_nSize;
  char*   m_pszSubject;
  size_t   m_nSubject;
  char*   m_pszFrom;
  size_t   m_nFrom;
  char*   m_pszMessageId;
  size_t   m_nMessageId;
  char*   m_pszDate;
  size_t   m_nDate;
  time_t   m_nUnixDate;
}XOVERINFO,*PXOVERINFO;
#endif

// The DLL must declare this variable and initialise it with function
// pointers to all of the entry points that Newsbin will be calling.
extern ENTRYPOINTS_V2 __declspec(dllexport) Entry;

//
//	Function Prototypes
//	These are provided as a convenience to the developer on the
//	assumption that he uses the suggested names for these routines.
//	Developers are at liberty to use their own names, but then they
//	need to define their own function prorotypes.
//

HRESULT   WINAPI PG_Init(PLUGIN_TYPE* pType, 
                       FILECALLBACKS_V1* p1, 
                       POSTCALLBACKS_V1* p2); 

HRESULT   WINAPI PG_Description(char* pszDescription, 
                              size_t nLength); 

HRESULT   WINAPI PG_Extension(char* pszExtension, 
                            size_t nLength); 

HRESULT   WINAPI PG_Options(HWND hWnd); 

HRESULT   WINAPI PG_Open(const char* pszFilename); 

HRESULT   WINAPI PG_Read(RECORD_T* pRecord); 
HRESULT   WINAPI PG_Close(void); 

HRESULT   WINAPI PG_PreDownload(NBFILEDATA hFileData); 
HRESULT   WINAPI PG_PostDownload(NBFILEDATA hFileData); 
HRESULT   WINAPI PG_ChunkDownload(size_t nIndex, 
                                const unsigned char* pszData, 
                                size_t nLength); 
HRESULT   WINAPI PG_PostFilter(NBPOSTDATA hPostData); 
HRESULT   WINAPI PG_FileFilter(NBFILEDATA hFileData); 

//
// New versioning mechanism. Plugin reports it's version and Newsbin reports it's version
// if the plugin thinks it'll work with this version it returns S_OK;
//
HRESULT   WINAPI PG_Version(VERSION_TYPE* PluginVersion,VERSION_TYPE nNewsbinVersion);

#ifdef __cplusplus
}
#endif

#endif